//---------------------------------------------------------------------------


#pragma hdrstop

#include "ControlQueue.h"
#include "UaMain.h"
#include "Mutex.h"
#include "ScopedLock.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

#include "fifo.h"
#include "Command.h"

namespace
{
	Fifo<Command, 512> fifo;
	Mutex mutex;
	void resetCmd(Command *cmd)
	{
		cmd->type = Command::NONE;
		cmd->target = "";
		cmd->accountId = -1;
		cmd->callId = -1;
		cmd->key = '\0';
		cmd->bEnabled = false;
	}
	int findAccIdForCall(int callId)
	{
		UaCalls &uaCalls = UaMain::Instance().uaCalls;
		UaCalls::CallData* data = uaCalls.FindCall(callId);
		if (data)
			return data->accId;
		return -1;
	}
}

ControlQueue::ControlQueue()
{

}

int ControlQueue::GetCommand(Command& cmd)
{
	ScopedLock<Mutex> lock(mutex);
	Command *tmpcmd = (Command*)fifo.getReadable();
	if (tmpcmd == NULL)
	{
		return 1;
	}
	cmd = *tmpcmd;
	fifo.pop();
	return 0;
}

void ControlQueue::Call(int accountId, int callId, AnsiString target)
{
	ScopedLock<Mutex> lock(mutex);
	Command *cmd = (Command*)fifo.getWriteable();
	if (!cmd)
		return;
	resetCmd(cmd);
	cmd->type = Command::CALL;
	cmd->accountId = accountId;
	cmd->callId = callId;
	cmd->target = target;
	fifo.push();
}

void ControlQueue::Answer(int callId)
{
	ScopedLock<Mutex> lock(mutex);
	Command *cmd = (Command*)fifo.getWriteable();
	if (!cmd)
		return;
	resetCmd(cmd);
	cmd->type = Command::ANSWER;
	cmd->callId = callId;
	cmd->accountId = findAccIdForCall(callId);
	fifo.push();
}

void ControlQueue::SendDigit(int callId, char key)
{
	ScopedLock<Mutex> lock(mutex);
	Command *cmd = (Command*)fifo.getWriteable();
	if (!cmd)
		return;
	resetCmd(cmd);
	cmd->type = Command::SEND_DIGIT;
	cmd->callId = callId;
	cmd->accountId = findAccIdForCall(callId);
	cmd->key = key;
	fifo.push();
}

void ControlQueue::SetMsgLogging(bool enabled)
{
	ScopedLock<Mutex> lock(mutex);
	Command *cmd = (Command*)fifo.getWriteable();
	if (!cmd)
		return;
	resetCmd(cmd);
	cmd->type = Command::SET_MSG_LOGGING;
	cmd->bEnabled = enabled;
	fifo.push();
}

void ControlQueue::Hangup(int callId)
{
	ScopedLock<Mutex> lock(mutex);
	Command *cmd = (Command*)fifo.getWriteable();
	if (!cmd)
		return;
	resetCmd(cmd);
	cmd->type = Command::HANGUP;
	cmd->callId = callId;
	cmd->accountId = findAccIdForCall(callId);
	fifo.push();
}


